# -*- coding: utf-8 -*-
import json

import numpy as np
from typing import Dict, List
from dspawpy.io.read import load_h5


def write_atoms(fileobj, hdf5):
    fileobj.write("DS-PAW Structure\n")
    fileobj.write("  1.00\n")
    lattice = np.asarray(hdf5["/AtomInfo/Lattice"]).reshape(-1,1)    # 将列表lattice下的多个列表整合

    fileobj.write("%10.6f %10.6f %10.6f\n" % (lattice[0], lattice[1], lattice[2]))
    fileobj.write("%10.6f %10.6f %10.6f\n" % (lattice[3], lattice[4], lattice[5]))
    fileobj.write("%10.6f %10.6f %10.6f\n" % (lattice[6], lattice[7], lattice[8]))

    elements = hdf5["/AtomInfo/Elements"]
    elements_set = []
    elements_number = {}
    for e in elements:
        if e in elements_set:
            elements_number[e] = elements_number[e] + 1
        else:
            elements_set.append(e)
            elements_number[e] = 1

    for e in elements_set:
        fileobj.write("  " + e)
    fileobj.write("\n")

    for e in elements_set:
        fileobj.write("%5d" % (elements_number[e]))
    fileobj.write("\n")
    if hdf5["/AtomInfo/CoordinateType"][0] == "Direct":
        fileobj.write("Direct\n")
    else:
        fileobj.write("Cartesian\n")
    for i, p in enumerate(hdf5["/AtomInfo/Position"]):
        fileobj.write("%10.6f" % p)
        if (i + 1) % 3 == 0:
            fileobj.write("\n")
    fileobj.write("\n")

def write_VESTA_format(hdf5:Dict, datakeys:str, filename):
    with open(filename, 'w') as file:
        write_atoms(file, hdf5)
        for key in datakeys:
            d = np.asarray(hdf5[key]).reshape(-1,1)     # 将列表hdf5[key]下的多个列表整合
            file.write("%5d %5d %5d\n" % tuple(hdf5["/AtomInfo/Grid"]))
            i = 0
            while i < len(d):
                for j in range(10):
                    file.write("%10.5f " % d[i])
                    i += 1
                    if i >= len(d):
                        break
                file.write("\n")

            file.write("\n")

def write_atoms_json(fileobj, atom_info):
    fileobj.write("DS-PAW Structure\n")
    fileobj.write("  1.00\n")
    lattice = atom_info["Lattice"]

    fileobj.write("%10.6f %10.6f %10.6f\n" % (lattice[0], lattice[1], lattice[2]))
    fileobj.write("%10.6f %10.6f %10.6f\n" % (lattice[3], lattice[4], lattice[5]))
    fileobj.write("%10.6f %10.6f %10.6f\n" % (lattice[6], lattice[7], lattice[8]))

    elements = [atom["Element"] for atom in atom_info["Atoms"]]
    elements_set = []
    elements_number = {}
    for e in elements:
        if e in elements_set:
            elements_number[e] = elements_number[e] + 1
        else:
            elements_set.append(e)
            elements_number[e] = 1

    for e in elements_set:
        fileobj.write("  " + e)
    fileobj.write("\n")

    for e in elements_set:
        fileobj.write("%5d" % (elements_number[e]))
    fileobj.write("\n")
    if atom_info["CoordinateType"] == "Direct":
        fileobj.write("Direct\n")
    else:
        fileobj.write("Cartesian\n")
    for atom in atom_info["Atoms"]:
        fileobj.write("%10.6f %10.6f %10.6f\n" % tuple(atom["Position"]))
    fileobj.write("\n")

def write_VESTA_format_json(atom_info: Dict, data: List, filename):
    with open(filename, 'w') as file:
        write_atoms_json(file, atom_info)
        for d in data:
            file.write("%5d %5d %5d\n" % tuple(atom_info["Grid"]))
            i = 0
            while i < len(d):
                for j in range(10):
                    file.write("%10.5f " % d[i])
                    i += 1
                    if i >= len(d):
                        break
                file.write("\n")

            file.write("\n")

def write_VESTA(in_filename:str, type, out_filename="DS-PAW.vesta"):
    """

    Args:
        in_filename:
        type:   "rho" / "Rho" / "RHO"
                "potential" / "Potential" / "POTENTIAL"
                "elf" / "Elf" / "ELF"
                "pcharge" / "Pcharge" / "PCHARGE"
                "boundcharge" / "BoundCharge" / "BOUNDCHARGE"
        out_filename:

    Returns:

    """
    if in_filename.endswith(".h5"):
        data = load_h5(in_filename)
        if type == "rho" or type == "Rho" or type == "RHO":
            write_VESTA_format(data, ["/Rho/TotalCharge"], out_filename)

        elif type == "potential" or type == "Potential" or type == "POTENTIAL":
            write_VESTA_format(data, ["/Potential/TotalElectrostaticPotential", "/Potential/TotalLocalPotential"], out_filename)

        elif type == "elf" or type == "Elf" or type == "ELF":
            write_VESTA_format(data, ["/ELF/TotalELF"], out_filename)

        elif type == "pcharge" or type == "Pcharge" or type == "PCHARGE":
            write_VESTA_format(data, ["/Pcharge/1/TotalCharge"], out_filename)

        elif type == "boundcharge" or type == "BoundCharge" or type == "BOUNDCHARGE":
            write_VESTA_format(data, ["/Rho"], out_filename)

        else:
            print(" -rho/potential/elf/pcharge- ONLY")
            return

    elif in_filename.endswith(".json"):
        with open(in_filename, 'r') as fin:
            data = json.load(fin)

        if type == "rho" or type == "Rho" or type == "RHO":
            write_VESTA_format_json(data["AtomInfo"], [data["Rho"]["TotalCharge"]], out_filename)

        elif type == "potential" or type == "Potential" or type == "POTENTIAL":
            write_VESTA_format_json(data["AtomInfo"], [data["Potential"]["TotalElectrostaticPotential"], data["Potential"]["TotalLocalPotential"]], out_filename)

        elif type == "elf" or type == "Elf" or type == "ELF":
            write_VESTA_format_json(data["AtomInfo"], [data["ELF"]["TotalELF"]], out_filename)

        elif type == "pcharge" or type == "Pcharge" or type == "PCHARGE":
            write_VESTA_format_json(data["AtomInfo"], [data["Pcharge"][0]["TotalCharge"]], out_filename)

        elif type == "boundcharge" or type == "BoundCharge" or type == "BOUNDCHARGE":
            write_VESTA_format_json(data["AtomInfo"], [data["Rho"]], out_filename)

        else:
            print(" -rho/potential/elf/pcharge- ONLY")
            return

    else:
        print("file - " + in_filename + " :  Unsupported format!")
        return
